import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;

import roombacomm.RoombaCommSerial;


public class BeeboXBoxDrive extends JFrame
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -6369395908604593638L;
	enum MoveState
	{
		Forward{ @SuppressWarnings("unused")
		public int songNum(){return 4;}},
		Backward{ @SuppressWarnings("unused")
		public int songNum(){return 5;}},
		Left{ @SuppressWarnings("unused")
		public int songNum(){return 6;}},
		Right{ @SuppressWarnings("unused")
		public int songNum(){return 7;}},
		None{ @SuppressWarnings("unused")
		public int songNum(){return 0;}},
	};
	
	enum RemoteState
	{
		Explore{ @SuppressWarnings("unused")
		public int songNum(){ return 8; }},
		Remote{ @SuppressWarnings("unused")
		public int songNum(){ return 9; }},
	}
	
	public static final int kDefaultMoveSpeed = 200;
	
	public class BeeboThread implements Runnable
	{
		BeeboThread( BeeboXBoxDrive robotTest )
		{
			pRobotTest = robotTest;
		}
		
		void updateExplore()
		{	
			if( mInput.isRemotePressed() )
			{
				setRemoteState(RemoteState.Remote);
				return;
			}
			
			  if( mRoombaComm.bumpLeft() ) 
			  {
				  mRoombaComm.playSong(3);
				  mRoombaComm.spinRight(90);
		      }
		      else if( mRoombaComm.bumpRight() ) 
		      {
		    	  mRoombaComm.playSong(3);
		    	  mRoombaComm.spinLeft(90);
		      }
		      else if( mRoombaComm.wall() ) 
		      { 
		    	  mRoombaComm.playSong(3);
		      }
			  mRoombaComm.goForward();
		}
		
	
		void updateRemote()
		{
			boolean hasInput = false;				
			
			if (pRobotTest.mInput.forward())
			{			
				hasInput = true;
				pRobotTest.setMoveState(MoveState.Forward);
			}		
			else if( pRobotTest.mInput.backward() )
			{
				hasInput = true;
				pRobotTest.setMoveState(MoveState.Backward);
			}			
			
			if( pRobotTest.mInput.left() )
			{
				hasInput = true;
				pRobotTest.setMoveState(MoveState.Left);
			} else if ( pRobotTest.mInput.right() )
			{
				hasInput = true;
				pRobotTest.setMoveState(MoveState.Right);
			}
			
			if( pRobotTest.mInput.isExplorePressed() )
			{
				setRemoteState(RemoteState.Explore);
			}
			
			// no input has been recieved, stop the motors
			if( !hasInput )
			{
				pRobotTest.setMoveState(MoveState.None);
				pRobotTest.mRoombaComm.stop();
			}	
		}		

		@Override
		public void run() 
		{
			while( true )
			{
				if (!mInput.poll()) 				
				{
					continue;
				}				
				
				updateRemote();	
				
				// if input says start
				// disconnect and stop
				if( pRobotTest.mInput.isStartPressed() )
				{
					pRobotTest.mRoombaComm.stop();
					break;			
				}	
				
				pRobotTest.mRoombaComm.updateSensors();
				
				Thread.yield();
			}
			
			pRobotTest.shutdown();
		}
		
		private BeeboXBoxDrive pRobotTest;
	}



	public BeeboXBoxDrive() 
	{
		mInput = new InputModule();
		mMoveState = MoveState.None;
		mRemoteState = RemoteState.Remote;
	}
	
	static final int kStartNote = 60;
	
	void setMoveState(MoveState newState)
	{
		if( mMoveState == newState )
			return;
		
		mMoveState = newState;
		
		if( newState == MoveState.None )
			return;
		
		int songNum = 0;
		
		if( newState == MoveState.Forward )
		{
			songNum = 4;
			mRoombaComm.goForward();
		}
		else if ( newState == MoveState.Backward )
		{
			songNum = 5;
			mRoombaComm.goBackward();
		}
		else if ( newState == MoveState.Left )
		{
			songNum = 6;
			mRoombaComm.spinLeft();
		}
		else if ( newState == MoveState.Right )
		{
			songNum = 7;
			mRoombaComm.spinRight();
		}
	
		mRoombaComm.playSong(songNum);
	}
	
	void sendSong(int songnum, String phrase)
	{
		// convert phrase to song		
		int length = phrase.length();
		
		if( length > 16 )
			length = 16;
		
		int[] song = new int[length*2];
		
		for( int i = 0; i < length; i++)
		{
			char c = phrase.charAt(i);
			
			int note = Character.getNumericValue(c) + kStartNote;
			
			song[i*2] = note;
			song[i*2 + 1] = 10;
		}	
		
		// create song
		mRoombaComm.createSong(songnum, song);
		
		mRoombaComm.pause(200);
	}
	
	void init()
	{
		// setup input (xbox controller)
		mInput.setup();
		
		// setup and connect to Roomba
		String portname = "COM3";
		mRoombaComm = new RoombaCommSerial();
        
		mRoombaComm.setPortname(portname);
		mRoombaComm.setProtocol("SCI"); 

        if( ! mRoombaComm.connect( portname ) ) 
        {
            System.out.println("Couldn't connect to "+portname);
            return;
        }      
	        
        System.out.println("Roomba startup");
        mRoombaComm.startup();
        mRoombaComm.control();
        mRoombaComm.pause(100);

        mRoombaComm.updateSensors();
        
        mRoombaComm.setSpeed(kDefaultMoveSpeed);
        
        sendSong(0, "Initializing");
        
        mRoombaComm.playSong(0);
        
        mRoombaComm.pause(1000);
        
        sendSong(1, "Hello");
        sendSong(2, "Goodbye");
        sendSong(3, "ExcuseMe");
        sendSong(4, "Forward");
        sendSong(5, "Backward");
        sendSong(6, "Right");
        sendSong(7, "Left");
        sendSong(8, "Explore");
        sendSong(9, "Remote");
        
        mRoombaComm.playSong(1);
        
        mRoombaComm.pause(1000);  
        
	}
	
	
	private void setRemoteState(RemoteState newState) 
	{

		if( mRemoteState == newState)
			return;
		
		setMoveState(MoveState.None);
		mRoombaComm.stop();
		
		int songNum = 0;
		
		if( newState == RemoteState.Explore )
			songNum = 8;
		else if ( newState == RemoteState.Remote )
			songNum = 9;
		
		mRoombaComm.playSong(songNum);
		mRoombaComm.pause(200);
		
		mRemoteState = newState;		
	}

	public void run()
	{	
	      mBeeboThread = 
			new BeeboThread(this);
	      Thread t = new Thread(mBeeboThread);
	      t.start();
	}
	
	void shutdown()
	{
		mRoombaComm.playSong(2);
		mRoombaComm.pause(1000);
		
		mRoombaComm.powerOff();
		mRoombaComm.pause(100);
		
		mRoombaComm.disconnect();
		
		System.out.println("Done");
	}
	
	
	InputModule mInput;
	RoombaCommSerial mRoombaComm;
	MoveState mMoveState;
	RemoteState mRemoteState;
	BeeboThread mBeeboThread;
	
	public static void main(String[] args) 
	{
		BeeboXBoxDrive frame = 
			new BeeboXBoxDrive();
	
		frame.setTitle("Roomba Drive!");
        WindowListener l = new WindowAdapter() 
        {
            public void windowClosing(WindowEvent e) 
            {
            	System.exit(0);
            }
        };
        
		frame.addWindowListener(l);
		frame.pack();
		frame.setVisible(true);
		
		frame.init();
		frame.run();
	}
}
